C LAST UPDATE: 
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C**************************
C**     LFN_UNQ_GET      **
C**************************
C
CD-Routine: Get Reflection Counts - LFN_UNQ_GET
C This routine returns the number of times a reflection has been registerd
C as being present in various categories in the unique reflections list
C whose counts have been updated by calls to the LFN_UNQ_INC routine.
CD-end
CD-Fortran:
      SUBROUTINE LFN_UNQ_GET (IREF, NSING, NMULT, NDOUBLE, NTRIPLE, 
     +                        NHIGHER, NPLUS, NMINUS, NCLEAR, NSPAT)
CD-end
      INCLUDE 'implicit_none.finc'
C
C Purpose: Get counts details for a reflection in the unique reflections list
C
C          (After counts set via calls to the LFN_UNQ_INC routine)
C
C Author:  John W. Campbell, September 1993
C
C Arguments:
C
      INTEGER IREF
      INTEGER NSING
      INTEGER NMULT
      INTEGER NDOUBLE
      INTEGER NTRIPLE
      INTEGER NHIGHER
      INTEGER NPLUS
      INTEGER NMINUS
      INTEGER NCLEAR
      INTEGER NSPAT
CD-Parameters: 
C  IREF      i (R)   Reflection number in the list
C  NSING     i (W)   No. of occurrences as a single (returns -1 if error 
C                    in IREF)
C  NMULT     i (W)   No. of occurrences as a multiple of any kind
C  NDOUBLE   i (W)   No. of occurrences as part of a double
C  NTRIPLE   i (W)   No. of occurrences as part of a multiple
C  NHIGHER   i (W)   No. of occurrences as part of a higher multiple
C  NPLUS     i (W)   No. of I+ occurrences
C  NMINUS    i (W)   No. of I- occurrences
C  NCLEAR    i (W)   No. of occurrences as a non spatially overlapped 
C                    reflection    
C  NSPAT     i (W)   No. of occurrences as a spatially overlapped 
C                    reflection
CD-end
C====== GLOBALS:
C
      EXTERNAL LFN_UNQ_BDAT
      INCLUDE 'lfn_unq.finc'
C
C====== FUNCTION TYPES:
C

C
C====== LOCALS:
C
      INTEGER I
      INTEGER NTWO20
      INTEGER NTWO10
      INTEGER INDX

C     INTEGER I           !Temp. integer value
C     INTEGER NTWO20      !2**20
C     INTEGER NTWO10      !2**10
C     INTEGER INDX        !Index into reflections list

      DATA NTWO20/1048576/
      DATA NTWO10/1024/

C
C-------------------------------------------------------------------------------
C
      IF (IREF.LT.1.OR.IREF.GT.NUM_UNQ) THEN
         NSING = -1
         NMULT = 0
         NDOUBLE = 0
         NTRIPLE = 0
         NHIGHER = 0
         NPLUS = 0
         NMINUS = 0
         NCLEAR = 0
         NSPAT = 0
         RETURN
      ELSE
         INDX = INDX_UNQ(IREF)
         I = COUNTS(INDX)
         NSING = I/NTWO20
         I = I - NTWO20*NSING
         NMULT = I/NTWO10
         NSPAT = I - NTWO10*NMULT
         I = MULT_CNT(INDX)
         NDOUBLE = I/NTWO20
         I = I - NTWO20*NDOUBLE
         NTRIPLE = I/NTWO10
         NHIGHER = I - NTWO10*NTRIPLE
         I = SIGN_CNT(INDX)
         NPLUS = I/NTWO10
         NMINUS = I - NTWO10*NPLUS
         NCLEAR = NSING + NMULT - NSPAT
      END IF
      RETURN
      END
